package test;

import cn.banny.unidbg.Module;
import cn.banny.unidbg.arm.ARMEmulator;
import cn.banny.unidbg.linux.android.AndroidARMEmulator;
import cn.banny.unidbg.linux.android.AndroidResolver;
import cn.banny.unidbg.linux.android.dvm.*;
import cn.banny.unidbg.linux.android.dvm.array.ByteArray;
import cn.banny.unidbg.memory.Memory;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;

public class MpaasCryptoClient extends AbstractJni {

    public static void main(String[] args) throws IOException {
        final MpaasCryptoClient client = new MpaasCryptoClient();

        final boolean init = client.init(null,
                "-----BEGIN PUBLIC KEY-----\n" +
                        "MFYwEAYHKoZIzj0CAQYFK4EEAAoDQgAEzuD6Rgl5vZQfXDbOoACTLKmmJimg7YPk\n" +
                        "PsczN3ZG3z2UcuQk+p7429L+ZaeSj9JLyvMK1yxzXjgWEYbk+L+Vqw==\n" +
                        "-----END PUBLIC KEY-----"
        );

        System.out.println(init);

        final byte[][] encode = client.encode("G07CIMkel8cJunRc".getBytes(), "data".getBytes(), 1);

    }

    private ARMEmulator mEmulator;
    private VM mVM;
    private DvmObject mProxy;
    private Module mModule;

    public MpaasCryptoClient() throws IOException {

        mEmulator = new AndroidARMEmulator(getClass().getName());
        final Memory memory = mEmulator.getMemory();
        memory.setLibraryResolver(new AndroidResolver(19));
        memory.setCallInitFunction();

        mVM = mEmulator.createDalvikVM(null);
        mVM.setJni(this);
        final DalvikModule dm = mVM.loadLibrary(new File("src/test/resources/so/libmpaas_crypto.so"), true);
        dm.callJNI_OnLoad(mEmulator);
        mModule = dm.getModule();

        mProxy = mVM.resolveClass("com/alipay/mobile/common/mpaas_1crypto/Client").newObject(null);
    }

    public boolean init(String priKey, String pubKey) {
        // MpaasNetConfigUtil.getAsymmetricEncryptMethod(Context) return 2
        Number ret = mProxy.callJniMethod(mEmulator, "init(Ljava/lang/String;Ljava/lang/String;)Z",
                null,
                mVM.addLocalObject(new StringObject(mVM, pubKey))
        );
        return ret.intValue() == 1;
    }

    public byte[][] encode(byte[] bArr, byte[] bArr2, int i) {

        //mEmulator.attach(DebuggerType.GDB_SERVER);

        final Number ret = mProxy.callJniMethod(mEmulator, "encode([B[BI)[[B",
                mVM.addLocalObject(new ByteArray(bArr)),
                mVM.addLocalObject(new ByteArray(bArr2)),
                i
        );
        final DvmObject object = mVM.getObject(ret.intValue() & 0xffffffffL);
        System.out.println(object);
        return null;
    }

    private long value = 0;

    @Override
    public long getLongField(BaseVM vm, DvmObject dvmObject, String signature) {
        if ("com/alipay/mobile/common/mpaas_1crypto/Client->handle:J".equals(signature)) {
            return value;
        }
        return super.getLongField(vm, dvmObject, signature);
    }

    @Override
    public void setLongField(BaseVM vm, DvmObject dvmObject, String signature, long value) {
        if ("com/alipay/mobile/common/mpaas_1crypto/Client->handle:J".equals(signature)) {
            this.value = value;
            return;
        }
        super.setLongField(vm, dvmObject, signature, value);
    }

    private static void addLibraryDir(String libraryPath) throws IOException {
        try {
            Field field = ClassLoader.class.getDeclaredField("usr_paths");
            field.setAccessible(true);
            String[] paths = (String[]) field.get(null);
            for (String path : paths) {
                if (libraryPath.equals(path)) {
                    return;
                }
            }

            String[] tmp = new String[paths.length + 1];
            System.arraycopy(paths, 0, tmp, 0, paths.length);
            tmp[paths.length] = libraryPath;
            field.set(null, tmp);
        } catch (IllegalAccessException e) {
            throw new IOException(
                    "Failedto get permissions to set library path");
        } catch (NoSuchFieldException e) {
            throw new IOException(
                    "Failedto get field handle to set library path");
        }
    }

    static {
        System.setProperty("jna.library.path", "prebuilt/win32");
        try {
            addLibraryDir("prebuilt/win32");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
